// Some common function definition

#include "Common.h"

// Write a byte out to the specified port.
void outb(u16int port, u8int value)
{
    asm volatile ("outb %1, %0" : : "dN" (port), "a" (value));
}

// Take a byte from the specified port
u8int inb(u16int port)
{
   u8int ret;
   asm volatile("inb %1, %0" : "=a" (ret) : "dN" (port));
   return ret;
}

// Take a word from the specified port
u16int inw(u16int port)
{
   u16int ret;
   asm volatile ("inw %1, %0" : "=a" (ret) : "dN" (port));
   return ret;
} 


// Copy len number of elements from source to destination
void memcpy(u8int *dest, const u8int *src, u32int len)
{
   u8int *dp = (u8int*)dest;
   const u8int *sp = (const u8int*)src;	
   for(; len != 0; len--) *dp++ = *sp++;
}

// Write len copies of val into destination
void memset(u8int *dest, u8int val, u32int len)
{
   u8int *dp = (u8int*)dest;
   for(; len != 0; len--) *dp++ = val;
}

// Write len copies of val into destination
void memsetw(u16int *dest, u16int val, u32int len)
{
   u16int *dp = (u16int*)dest;
   for(; len != 0; len--) *dp++ = val;
}

// Compares two strings. Returns -1 if 
// str1 < str2, 0 if they are equal or 1 otherwise.
int strcmp(char *str1, char *str2)
{
    int i = 0;
    int failed = 0;
    while(str1[i] != '\0' && str2[i] != '\0')
    {
        if(str1[i] != str2[i])
        {
            failed = 1;
            break;
        }
        i++;
    }
    // why did the loop exit?
    if( (str1[i] == '\0' && str2[i] != '\0') || (str1[i] != '\0' && str2[i] == '\0') )
        failed = 1;

    return failed;
}

// Concatenate the NULL-terminated string src onto
// the end of dest, and return dest.
char *strcat(char *dest, const char *src)
{
    char *dp = (char*)dest;
    char *sp = (char*)src;
    while (*dest != '\0')
    {
        *dp++;
    }

    do
    {
        *dp++ = *sp++;
    }
    while (*sp != '\0');
    return dest;
}

// Copy the NULL-terminated string src into dest, and
// return dest.
char *strcpy(char *dest, const char *src)
{
   char *dp = (char*)dest;
   char *sp = (char*)src;
   do
   {
      *dp++ = *sp++;
   }
   while(*sp != '\0');
   return dest;
}

