// common.h -- Defines typedefs and some global functions.
//             From JamesM's kernel development tutorials.

#ifndef COMMON_H
#define COMMON_H

// Some nice typedefs, to standardise sizes across platforms.
// These typedefs are written for 32-bit X86.
typedef unsigned int   u32int;	// unsigned int = unsigned 32 bit integer
typedef          int   s32int;	// int = signed 32 bit integer
typedef unsigned short u16int;	// unsigned short int = unsigned 16 bit integer
typedef          short s16int;	// short = signed 16 bit integer
typedef unsigned char  u8int;	// unsigned char = unsigned 8 bit integer
typedef          char  s8int;	// char = signed 8 bit integer

/* This defines what the stack looks like after an ISR was running */
struct regs
{
    unsigned int gs, fs, es, ds;      /* pushed the segs last */
    unsigned int edi, esi, ebp, esp, ebx, edx, ecx, eax;  /* pushed by 'pusha' */
    unsigned int int_no, err_code;    /* our 'push byte #' and ecodes do this */
    unsigned int eip, cs, eflags, useresp, ss;   /* pushed by the processor automatically */ 
};

// Write a byte out to the specified port.
void outb(u16int port, u8int value);

// Read a byte from the specified port.
u8int inb(u16int port);

// Read a word from the specified port.
u16int inw(u16int port);

// Copy len bytes from src to dest.
void memcpy(u8int *dest, const u8int *src, u32int len);

// Write len copies of val into dest.
void memset(u8int *dest, u8int val, u32int len);

// Compare two strings. Should return -1 if 
// str1 < str2, 0 if they are equal or 1 otherwise.
int strcmp(char *str1, char *str2);

// Copy the NULL-terminated string src into dest, and
// return dest.
char *strcpy(char *dest, const char *src);

// Concatenate the NULL-terminated string src onto
// the end of dest, and return dest.
char *strcat(char *dest, const char *src);

#endif // COMMON_H
