#ifndef GDT_H
#define GDT_H

#include "Common.h"

/* Defines a GDT entry.*/ 
struct gdt_entry
{
    u16int limit_low;
    u16int base_low;
    u8int base_middle;
    u8int access;
    u8int granularity;
    u8int base_high;
} __attribute__((packed));

// Special pointer to be loaded. 
struct gdt_ptr
{
    u16int limit;
    u32int base;
} __attribute__((packed));

// Our GDT, with 3 entries, and finally our special GDT pointer 
struct gdt_entry gdt[3];
struct gdt_ptr gp;

// This will be a function in LGdt.s. We use this to properly reload the new segment registers 
extern void gdt_flush();

// Should be called by main. This will setup the special GDT
// pointer, set up the first 3 entries in our GDT, and then
// finally call gdt_flush() in our assembler file in order
// to tell the processor where the new GDT is and update the
//  new segment registers 
void gdt_install();

// Setup a descriptor in the Global Descriptor Table 
void gdt_set_gate(int num, unsigned long base, unsigned long limit, unsigned char access, unsigned char gran);

#endif
