// main.c -- Defines the C-code kernel entry point, calls initialisation routines.

#include "Monitor.h"
#include "Timer.h"
#include "Idt.h"
#include "Isr.h"
#include "Gdt.h"
#include "Irq.h"
#include "Keyboard.h"
#include "Cli.h"
#include "Kheap.h"
#include "Paging.h"

int main(struct multiboot *mboot_ptr)
{
    char c = 'n';
    monitor_init(1, 15);			// Initializes monitor and video display with attributes, white on blue.
    
    monitor_write_on_line("SOS - Shikhin's Operating System", 0, 7, 15);		// Prints the message on the first line, with grey background.
    monitor_write("Monitor initialzed at text mode.\n");

    gdt_install();				// Installs gdt.
    idt_install();				// Installs idt.
    isr_install();				// Installs isr.
    irq_install();				// Installs irq.
    
    asm volatile("sti");

    monitor_write("\nGlobal Descriptor Table initialized.\n");
    monitor_write("Interrupt Descriptor Table initialized.\n");
    monitor_write("Interrupt Service Routines initialized.\n");
    monitor_write("Interrupt Requests initialized.\n");
    
    paging_install();				// Installs paging.
    monitor_write("Paging initialized.\n");

    timer_install(); 				// Installs PIT.
    monitor_write("\nProgrammable Interval Timer initialized at ");
    monitor_write_dec(speed);
    monitor_write(" Hz.\n");

    keyboard_install();      			// Installs keyboard.
    monitor_write("\nKeyboard initialized, with basic keys, excluding:\nShift, Ctrl, Alt, Caps Lock, Tab, Function Keys, Esc, Delete, Backspace and Arrow keys\n\n");  

    monitor_write("Kernel initialzed.\n");
    monitor_write("Command Line Interface initialized. New users type 'help'.\n\n");

    u32int *ptr = (u32int*)0xA0000000;
    u32int do_page_fault = *ptr;

    cli();					// Initializes command line interface.
}
