#include "Monitor.h"
#include "Common.h"
#include "Timer.h"
#include "Irq.h"

/* This will keep track of how many ticks that the system
*  has been running for */
int timer_ticks = 0;

/* This will keep track of the seconds passed */
int seconds;

/* Handles the timer. We
*  increment the 'timer_ticks' variable every time the
*  timer fires. By default, the timer fires 18.222 times
*  per second. */
void timer_handler(struct regs *r)
{
    /* Increment our 'tick count' */
    timer_ticks++;

    /* Every 18 clocks (approximately 1 second), we will
    *  display a message on the screen */
    if (timer_ticks % speed == 0)
    {
        seconds++;
    }
}

/* Sets up the system clock by installing the timer handler
*  into IRQ0 */
void timer_install()
{
    /* Installs 'timer_handler' to IRQ0 */
    irq_install_handler(0, timer_handler);

    speed = 18.111; seconds = 0;
}

/* This will cause the system to wait untill the given ticks has
*  been reached */
void timer_wait(int ticks)
{
    unsigned long eticks;

    eticks = timer_ticks + ticks;
    while(timer_ticks < eticks);
}

/* This will show, how many seconds have passed, since the timer
*  has been initialized */
int show_seconds()
{
    return seconds;
}
